.panel-body.empty-message {
  padding-block: 0.5rem;
  padding-inline: 1rem;
  text-align: center;
}

.cockpit-log-panel .panel-body {
  padding-inline: 0;
}

.cockpit-log-panel .pf-v5-c-card__header-main > .pf-v5-c-card__title > .pf-v5-c-card__title-text {
  padding: 0;
  font-weight: normal;
  font-size: var(--pf-v5-global--FontSize--2xl);
}
