.podman-search .pf-v5-c-modal-box__body {
    display: grid;
    grid-auto-flow: row;
    overflow: hidden;
    grid-template-rows: auto auto 1fr;
    grid-gap: var(--pf-v5-global--spacer--sm);
}

.image-list-item {
    display: grid;
    grid-gap: 0 1rem;
}

.image-list-item + .image-list-item {
    border-block-start: 1px solid var(--pf-v5-global--BorderColor--200);
}

.image-list-item > .image-name {
    color: var(--pf-v5-global--Color--100);
}

@media (min-width: 768px) {
    .image-list-item {
        grid-template-columns: 1fr max-content;
    }
}

.podman-search .image-search-modal-footer-grid {
    display: grid;
    grid-template-columns: 1fr auto auto;
    grid-gap: 0.25rem;
}

.image-tag-entry {
    max-inline-size: 15rem;
}

@media (max-width: 340px) {
    /* Shrink buttons to accommodate iPhone 5/SE */
    .podman-search .modal-footer > .btn {
        padding-inline: 0.25rem;
    }
}

.image-search-tag-form {
    margin-block-end: var(--pf-v5-global--spacer--md);
}

.podman-search .pf-v5-c-modal-box__footer {
    display: initial;
}

.podman-search .pf-v5-c-data-list {
    overflow-y: auto;
}

.podman-search .pf-v5-l-flex .pf-v5-c-form__group:nth-child(2) {
    grid-template-columns: 2rem var(--pf-v5-c-form--m-horizontal__group-control--md--GridColumnWidth);
}
