#shutdown-group {
  overflow: visible;
}

.shutdown-delay-group {
  // Add spacing between rows for when the flex items wrap
  row-gap: var(--pf-v5-global--spacer--sm);

  .shutdown-time-picker {
    max-inline-size: 7rem;
  }

  .shutdown-select-delay,
  .shutdown-date-picker {
    max-inline-size: 10rem;
  }
}
