// Required for `pf-v5-u-box-shadow-sm` as inline alert don't have a box shadow
@use "../../node_modules/@patternfly/patternfly/utilities/BoxShadow/box-shadow.css";

.pf-v5-c-page__main-section.ct-no-bottom-padding {
  --pf-v5-c-page__main-section--PaddingBottom: 0;
}

// <Toolbar> embedded into an <Alert>
.pf-v5-c-toolbar.ct-alert-toolbar {
  --pf-v5-c-toolbar--BackgroundColor: transparent;
  --pf-v5-c-toolbar--PaddingBottom: 0;
}
